searchNodes=[{"doc":"Validate parameters using a rules schema. The parameters can be any map, and can be string-based or atom-based. Goal uses the validation rules from in Ecto.Changeset , which means you can use any validation that is available for database fields for validating parameters with Goal. A common use-case is parsing and validating parameters from Phoenix controllers: defmodule MyApp.SomeController do import Goal @schema %{ id : [ format : :uuid , required : true ] , name : [ min : 3 , max : 20 , required : true ] } def create ( conn , params ) do with { :ok , attrs } &lt;- validate_params ( params , @schema ) do ... end end end Defining validations Define field types with :type : :string :integer :boolean :float :decimal :date :time :map {:array, inner_type} , where inner_type can be any of the field types See Ecto.Schema for the full list The default field type is :string . That means you don't have to define this field in the schema if the value will be a string. Define map fields with :properties . Define string validations: :equals , string value :is , string length :min , minimum string length :max , maximum string length :trim , boolean to remove leading and trailing spaces :squish , boolean to trim and collapse spaces :format , atom to define the regex (available are: :uuid , :email , :password , :url ) Define integer validations: :is , integer value :min , minimum integer value :max , maximum integer value :greater_than , minimum integer value :less_than , maximum integer value :greater_than_or_equal_to , minimum integer value :less_than_or_equal_to , maximum integer value :equal_to , integer value :not_equal_to , integer value Define enum validations: :excluded , list of disallowed values :included , list of allowed values :subset , list of values Bring your own regex Goal has sensible defaults for string format validation. If you'd like to use your own regex, e.g. for validating email addresses or passwords, you can add your own regex in your application configuration. config :goal , uuid_regex : ~r/^[[:alpha:]]+$/ , email_regex : ~r/^[[:alpha:]]+$/ , password_regex : ~r/^[[:alpha:]]+$/ , url_regex : ~r/^[[:alpha:]]+$/ Deeply nested maps Goal efficiently builds error changesets for nested maps. There is no limitation on depth. If the schema is becoming too verbose, you could consider splitting up the schema into reusable components. data = %{ &quot;nested_map&quot; =&gt; %{ &quot;map&quot; =&gt; %{ &quot;inner_map&quot; =&gt; %{ &quot;list&quot; =&gt; [ 1 , 2 , 3 ] } } } } schema = %{ nested_map : [ type : :map , properties : %{ inner_map : [ type : :map , properties : %{ map : [ type : :map , properties : %{ list : [ type : { :array , :integer } ] } ] } ] } ] } iex(1)&gt; data = %{ ... } iex(2)&gt; schema = %{ ... } iex(3)&gt; Goal . validate_params ( data , schema ) { :ok , %{ nested_map : %{ inner_map : %{ map : %{ list : [ 1 , 2 , 3 ] } } } } } Human-readable error messages Use Goal.traverse_errors/2 to build readable errors. Ecto and Phoenix by default use Ecto.Changeset.traverse_errors/2 , which works for embedded Ecto schemas but not for the plain nested maps used by Goal. def translate_errors ( changeset ) do Goal . traverse_errors ( changeset , &amp; translate_error / 1 ) end Credits This library is based on Ecto and I had to copy and adapt Ecto.Changeset.traverse_errors/2 . Thanks for making such an awesome library! 🙇","ref":"Goal.html","title":"Goal","type":"module"},{"doc":"Traverses changeset errors and applies the given function to error messages. Examples iex&gt; traverse_errors ( changeset , fn { msg , opts } -&gt; ...&gt; Regex . replace ( ~r&quot;%{( \\w +)}&quot; , msg , fn _ , key -&gt; ...&gt; opts |&gt; Keyword . get ( String . to_existing_atom ( key ) , key ) |&gt; to_string ( ) ...&gt; end ) ...&gt; end ) %{ title : [ &quot;should be at least 3 characters&quot; ] }","ref":"Goal.html#traverse_errors/2","title":"Goal.traverse_errors/2","type":"function"},{"doc":"Validates the parameters against a schema. Examples iex&gt; validate_params ( %{ &quot;email&quot; =&gt; &quot;jane@example.com&quot; } , %{ email : [ format : :email ] } ) { :ok , %{ email : &quot;jane@example.com&quot; } } iex&gt; validate_params ( %{ &quot;email&quot; =&gt; &quot;invalid&quot; } , %{ email : [ format : :email ] } ) { :error , % Ecto.Changeset { valid? : false , errors : [ email : { &quot;has invalid format&quot; , ... } ] } }","ref":"Goal.html#validate_params/2","title":"Goal.validate_params/2","type":"function"},{"doc":"","ref":"Goal.html#t:data/0","title":"Goal.data/0","type":"type"},{"doc":"","ref":"Goal.html#t:error/0","title":"Goal.error/0","type":"type"},{"doc":"","ref":"Goal.html#t:schema/0","title":"Goal.schema/0","type":"type"},{"doc":"Goal.Changeset contains an adapted version of Ecto.Changeset.traverse_errors/2 . You can use traverse_errors/2 to build all errors into a series of nested maps.","ref":"Goal.Changeset.html","title":"Goal.Changeset","type":"module"},{"doc":"Traverses changeset errors and applies the given function to error messages. This function is particularly useful when associations, embeds, maps and nested maps are cast in the changeset as it will traverse all associations, embeds, maps and nested maps and place all errors in a series of nested maps. A changeset is supplied along with a function to apply to each error message as the changeset is traversed. The error message function receives an error tuple {msg, opts} , for example: { &quot;should be at least %{count} characters&quot; , [ count : 3 , validation : :length , min : 3 ] } Examples iex&gt; traverse_errors ( changeset , fn { msg , opts } -&gt; ...&gt; Regex . replace ( ~r&quot;%{( \\w +)}&quot; , msg , fn _ , key -&gt; ...&gt; opts |&gt; Keyword . get ( String . to_existing_atom ( key ) , key ) |&gt; to_string ( ) ...&gt; end ) ...&gt; end ) %{ title : [ &quot;should be at least 3 characters&quot; ] } Optionally function can accept three arguments: changeset , field and error tuple {msg, opts} . It is useful whenever you want to extract validations rules from changeset.validations to build detailed error description. This function and documentation is gratefully copied and adapted from Ecto.Changeset.traverse_errors/2 🙇","ref":"Goal.Changeset.html#traverse_errors/2","title":"Goal.Changeset.traverse_errors/2","type":"function"},{"doc":"","ref":"Goal.Changeset.html#t:action/0","title":"Goal.Changeset.action/0","type":"type"},{"doc":"","ref":"Goal.Changeset.html#t:constraint/0","title":"Goal.Changeset.constraint/0","type":"type"},{"doc":"","ref":"Goal.Changeset.html#t:data/0","title":"Goal.Changeset.data/0","type":"type"},{"doc":"","ref":"Goal.Changeset.html#t:error/0","title":"Goal.Changeset.error/0","type":"type"},{"doc":"","ref":"Goal.Changeset.html#t:t/0","title":"Goal.Changeset.t/0","type":"type"},{"doc":"","ref":"Goal.Changeset.html#t:t/1","title":"Goal.Changeset.t/1","type":"type"},{"doc":"","ref":"Goal.Changeset.html#t:types/0","title":"Goal.Changeset.types/0","type":"type"},{"doc":"Defines regexes.","ref":"Goal.Regex.html","title":"Goal.Regex","type":"module"},{"doc":"Defines string operations.","ref":"Goal.String.html","title":"Goal.String","type":"module"},{"doc":"Returns a string where all leading, trailing, and multiple inner Unicode whitespaces have been removed. Examples iex&gt; squish ( &quot; banana man &quot; ) &quot;banana man&quot;","ref":"Goal.String.html#squish/1","title":"Goal.String.squish/1","type":"function"},{"doc":"A library for parsing and validating parameters. It takes the params (e.g. from an Phoenix controller action), validates them, and returns an atom-based map or an error changeset. It's based on Ecto , so every validation that you have for database fields can be applied in validating parameters. Goal is different from other validation libraries because of its syntax, it being Ecto-based, and it validates data using pure functions instead of building embedded Ecto.Schema in the background. Goal allows you to configure your own regexes. This is helpful in case of backward compatibility, where Goal's defaults might not match your production system's regexes.","ref":"readme.html","title":"Goal ⚽","type":"extras"},{"doc":"Add goal to the list of dependencies in mix.exs : def deps do [ { :goal , &quot;~&gt; 0.1.0&quot; } ] end","ref":"readme.html#installation","title":"Goal ⚽ - Installation","type":"extras"},{"doc":"There are several patterns you can choose to validate parameters using Goal. Using module attributes defmodule MyApp.SomeController do import Goal @schema %{ id : [ format : :uuid , required : true ] , name : [ min : 3 , max : 20 , required : true ] } def create ( conn , params ) do with { :ok , attrs } &lt;- validate_params ( params , @schema ) do ... end end end Using private functions defmodule MyApp.SomeController do import Goal def create ( conn , params ) do with { :ok , attrs } &lt;- validate_params ( params , schema ( ) ) do ... end end defp schema do %{ id : [ format : :uuid , required : true ] , name : [ min : 3 , max : 20 , required : true ] } end end","ref":"readme.html#usage","title":"Goal ⚽ - Usage","type":"extras"},{"doc":"Defining validations Define field types with :type : :string :integer :boolean :float :decimal :date :time :map {:array, inner_type} , where inner_type can be any of the field types See Ecto.Schema for the full list The default field type is :string . That means you don't have to define this field in the schema if the value will be a string. Define map fields with :properties . Define string validations: :equals , string value :is , string length :min , minimum string length :max , maximum string length :trim , boolean to remove leading and trailing spaces :squish , boolean to trim and collapse spaces :format , atom to define the regex (available are: :uuid , :email , :password , :url ) Define integer validations: :is , integer value :min , minimum integer value :max , maximum integer value :greater_than , minimum integer value :less_than , maximum integer value :greater_than_or_equal_to , minimum integer value :less_than_or_equal_to , maximum integer value :equal_to , integer value :not_equal_to , integer value Bring your own regex Goal has sensible defaults for string format validation. If you'd like to use your own regex, e.g. for validating email addresses or passwords, you can add your own regex in the configuration. config :goal , uuid_regex : ~r/^[[:alpha:]]+$/ , email_regex : ~r/^[[:alpha:]]+$/ , password_regex : ~r/^[[:alpha:]]+$/ , url_regex : ~r/^[[:alpha:]]+$/ Deeply nested maps Goal efficiently builds error changesets for nested maps. There is no limitation on depth. If the schema is becoming too verbose, you could consider splitting up the schema into reusable components. data = %{ &quot;nested_map&quot; =&gt; %{ &quot;map&quot; =&gt; %{ &quot;inner_map&quot; =&gt; %{ &quot;list&quot; =&gt; [ 1 , 2 , 3 ] } } } } schema = %{ nested_map : [ type : :map , properties : %{ inner_map : [ type : :map , properties : %{ map : [ type : :map , properties : %{ list : [ type : { :array , :integer } ] } ] } ] } ] } iex(1)&gt; data = %{ ... } iex(2)&gt; schema = %{ ... } iex(3)&gt; Goal . validate_params ( data , schema ) { :ok , %{ nested_map : %{ inner_map : %{ map : %{ list : [ 1 , 2 , 3 ] } } } } } Human-readable error messages Use Goal.traverse_errors/2 to build readable errors. Ecto and Phoenix by default use Ecto.Changeset.traverse_errors/2 , which works for embedded Ecto schemas but not for the plain nested maps used by Goal. def translate_errors ( changeset ) do Goal . traverse_errors ( changeset , &amp; translate_error / 1 ) end","ref":"readme.html#features","title":"Goal ⚽ - Features","type":"extras"},{"doc":"[x] Bring your own regex [x] ExDoc documentation [x] Basic syntax optimizations [ ] Macro for generating schemas using optional and required (like https://dry-rb.org/gems/dry-schema/1.10/ ) [ ] Release v0.1.0 on Hex.pm [ ] Convert incoming params from camelCase to snake_case","ref":"readme.html#roadmap","title":"Goal ⚽ - Roadmap","type":"extras"},{"doc":"This library is based on Ecto and I had to copy and adapt Ecto.Changeset.traverse_errors/2 . Thanks for making such an awesome library! 🙇","ref":"readme.html#credits","title":"Goal ⚽ - Credits","type":"extras"}]